@@ -10,12 +10,12 @@ from .extension.directive import (
 
 _version_file = _Path(_os.path.realpath(__file__)).parent / "VERSION"
 __version__ = _version_file.read_text().strip()
-state = SphinxCodeAutoLink()
 
 
 def setup(app: Sphinx):
     """Set up extension, directives and events."""
-    app.add_config_value('codeautolink_concat_blocks', 'none', 'html', types=[str])
+    state = SphinxCodeAutoLink()
+    app.add_css_file('sphinx-codeautolink.css')
     app.add_config_value('codeautolink_autodoc_inject', True, 'html', types=[bool])
 
     app.add_directive('concat-blocks', ConcatBlocks)
@@ -23,9 +23,9 @@ def setup(app: Sphinx):
     app.add_directive('implicit-import', ImplicitImport)
     app.add_directive('autolink-skip', AutoLinkSkip)
 
-    app.connect('builder-inited', state.read_references)
+    app.connect('builder-inited', state.build_inited)
+    app.connect('autodoc-process-docstring', state.autodoc_process_docstring)
     app.connect('doctree-read', state.parse_blocks)
     app.connect('doctree-resolved', state.generate_backref_tables)
     app.connect('build-finished', state.apply_links)
-    app.connect('autodoc-process-docstring', state.autodoc_process_docstring)
     return {'version': __version__}
