@@ -9,17 +9,18 @@ from dataclasses import dataclass
 from bs4 import BeautifulSoup
 from docutils import nodes
 
-from ..parse import parse_names, Name
+from ..parse import parse_names, Name, NameBreak
 from .backref import CodeExample
 from .directive import ConcatBlocksMarker, ImplicitImportMarker, AutoLinkSkipMarker
 
 
 @dataclass
-class SourceTransforms:
+class SourceTransform:
     """Transforms on source code."""
 
     source: str
     names: List[Name]
+    example: CodeExample
 
 
 class ParsingError(Exception):
@@ -33,35 +34,34 @@ class UserError(Exception):
 class CodeBlockAnalyser(nodes.SparseNodeVisitor):
     """Transform literal blocks of Python with links to reference documentation."""
 
-    def __init__(self, *args, concat_default: str, source_dir: str, **kwargs):
+    def __init__(self, *args, source_dir: str, **kwargs):
         super().__init__(*args, **kwargs)
-        self.code_refs = {}
+        self.source_transforms: List[SourceTransform] = []
         relative_path = Path(self.document['source']).relative_to(source_dir)
         self.current_document = str(relative_path.with_suffix(''))
         self.title_stack = []
         self.current_refid = None
-        self.source_transforms: List[SourceTransforms] = []
         self.implicit_imports = []
-        if concat_default not in ('none', 'section', 'file'):
-            raise UserError(
-                f'Invalid concatenation default value in "conf.py": `{concat_default}`'
-            )
-        self.concat_default = concat_default
-        self.concat_current = None
+        self.concat_global = False
+        self.concat_section = False
         self.concat_sources = []
         self.autolink_skip = None
 
     def unknown_visit(self, node):
         """Handle and delete custom directives, ignore others."""
         if isinstance(node, ConcatBlocksMarker):
-            if node.level not in ('none', 'section', 'file', 'reset'):
+            if node.mode not in ('off', 'section', 'on'):
                 raise UserError(
-                    f'Invalid concatenation argument: `{node.level}` '
+                    f'Invalid concatenation argument: `{node.mode}` '
                     f'in document "{self.current_document}"'
                 )
 
             self.concat_sources = []
-            self.concat_current = node.level if node.level != 'reset' else None
+            if node.mode == 'section':
+                self.concat_section = True
+            else:
+                self.concat_section = False
+                self.concat_global = (node.mode == 'on')
             node.parent.remove(node)
         elif isinstance(node, ImplicitImportMarker):
             if '\n' in node.content:
@@ -72,20 +72,22 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
             self.implicit_imports.append(node.content)
             node.parent.remove(node)
         elif isinstance(node, AutoLinkSkipMarker):
-            assert node.level in ('next', 'section', 'file', 'none')
-            self.autolink_skip = node.level if node.level != 'none' else None
+            if node.level not in ('next', 'section', 'file', 'off'):
+                raise UserError(
+                    f'Invalid skipping argument: `{node.level}` '
+                    f'in document "{self.current_document}"'
+                )
+            self.autolink_skip = node.level if node.level != 'off' else None
             node.parent.remove(node)
 
-    def _concat_mode(self) -> str:
-        return self.concat_current or self.concat_default
-
     def unknown_departure(self, node):
         """Ignore unknown nodes."""
 
     def visit_title(self, node):
         """Track section names and break concatenation and skipping."""
         self.title_stack.append(node.astext())
-        if self._concat_mode() == 'section':
+        if self.concat_section:
+            self.concat_section = False
             self.concat_sources = []
         if self.autolink_skip == 'section':
             self.autolink_skip = None
@@ -103,6 +105,10 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         implicit_imports = self.implicit_imports
         self.implicit_imports = []
 
+        skip = self.autolink_skip
+        if skip == 'next':
+            self.autolink_skip = None
+
         if (
             len(node.children) != 1
             or not isinstance(node.children[0], nodes.Text)
@@ -110,12 +116,14 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         ):
             return
 
+        example = CodeExample(
+            self.current_document, self.current_refid, list(self.title_stack)
+        )
         source = node.children[0].astext()
+        transform = SourceTransform(source, [], example)
+        self.source_transforms.append(transform)
 
-        if self.autolink_skip:
-            self.source_transforms.append(SourceTransforms(source, []))
-            if self.autolink_skip == 'next':
-                self.autolink_skip = None
+        if skip:
             return
 
         modified_source = '\n'.join(self.concat_sources + implicit_imports + [source])
@@ -136,13 +144,9 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
                 name.lineno -= hidden_len
                 name.end_lineno -= hidden_len
 
-        if self._concat_mode() != 'none':
+        if self.concat_section or self.concat_global:
             self.concat_sources.extend(implicit_imports + [source])
 
-        transforms = SourceTransforms(source, [])
-        example = CodeExample(
-            self.current_document, self.current_refid, list(self.title_stack)
-        )
         for name in names:
             if name.lineno < 1:
                 continue  # From concatenated source
@@ -150,19 +154,17 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
             if name.lineno != name.end_lineno:
                 msg = (
                     'sphinx-codeautolinks: multiline names are not supported, '
-                    f'found `{name.used_name}` in document {self.current_document} '
+                    f'found `{name.code_str}` in document {self.current_document} '
                     f'on lines {name.lineno} - {name.end_lineno}'
                 )
                 warn(msg, RuntimeWarning)
                 continue
 
-            transforms.names.append(name)
-            self.code_refs.setdefault(name.import_name, []).append(example)
-        self.source_transforms.append(transforms)
+            transform.names.append(name)
 
 
 def link_html(
-    document: Path, out_dir: str, transforms: List[SourceTransforms], inventory: dict
+    document: Path, out_dir: str, transforms: List[SourceTransform], inventory: dict
 ):
     """Inject links to code blocks on disk."""
     text = document.read_text('utf-8')
@@ -171,10 +173,24 @@ def link_html(
     inners = [block.select('div > pre')[0] for block in blocks]
 
     up_lvls = len(document.relative_to(out_dir).parents) - 1
-    link_pattern = '<a href="' + '../' * up_lvls + '{link}" title="{title}">{text}</a>'
+    link_pattern = (
+        '<a href="' + '../' * up_lvls
+        + '{link}" title="{title}" class="sphinx-codeautolink-a">{text}</a>'
+    )
     name_pattern = '<span class="n">{name}</span>'
     period = '<span class="o">.</span>'
 
+    # Expression asserts no dots before or after content nor a link after,
+    # i.e. a self-contained name or attribute that hasn't been linked yet
+    # so we are free to replace any occurrences, since the order of
+    # multiple identical replacements doesn't matter.
+    base_ex = r'(?<!<span class="o">\.</span>){content}(?!(<span class="o">\.)|(</a>))'
+    # Potentially instead assert an initial closing parenthesis followed by a dot
+    call_ex = (
+        r'(?<=\)</span><span class="o">\.</span>)'
+        r'{content}(?!(<span class="o">\.)|(</a>))'
+    )
+
     for trans in transforms:
         for ix in range(len(inners)):
             if trans.source == ''.join(inners[ix].strings).rstrip():
@@ -190,25 +206,18 @@ def link_html(
 
         lines = str(inner).split('\n')
 
-        # Expression asserts no dots before or after content nor a link after,
-        # i.e. a self-contained name or attribute that hasn't been linked yet
-        # so we are free to replace any occurrences, since the order of
-        # multiple identical replacements doesn't matter.
-        ex = r'(?<!<span class="o">\.</span>){content}(?!(<span class="o">\.)|(</a>))'
         for name in trans.names:
-            if name.import_name not in inventory:
-                continue
-
             html = period.join(
-                name_pattern.format(name=part) for part in name.used_name.split('.')
+                name_pattern.format(name=part) for part in name.code_str.split('.')
             )
             line = lines[name.lineno - 1]
 
             # Reverse because a.b = a.b should replace from the right
+            ex = call_ex if name.previous == NameBreak.call else base_ex
             matches = list(re.finditer(ex.format(content=html), line))[::-1]
             if not matches:
                 msg = (
-                    f'Could not match transformation of `{name.used_name}` '
+                    f'Could not match transformation of `{name.code_str}` '
                     f'on source line {name.lineno} in document "{document}", '
                     f'source:\n{trans.source}'
                 )
@@ -216,8 +225,8 @@ def link_html(
                 continue
 
             link = link_pattern.format(
-                link=inventory[name.import_name],
-                title=name.import_name,
+                link=inventory[name.resolved_location],
+                title=name.resolved_location,
                 text=html
             )
             start, end = matches[0].span()
